/**
 * @file Select
 * @description
 * @author fex
 * @date 2017-11-07
 */
/// <reference types="hoist-non-react-statics" />
import React from 'react';
import { ControllerStateAndHelpers } from 'downshift';
import { ClassNamesFn, ThemeProps } from '../theme';
import { Api } from '../types';
import { LocaleProps } from '../locale';
export interface Option {
    label?: string;
    scopeLabel?: string;
    value?: any;
    disabled?: boolean;
    children?: Options;
    visible?: boolean;
    hidden?: boolean;
    description?: string;
    defer?: boolean;
    deferApi?: Api;
    loading?: boolean;
    loaded?: boolean;
    [propName: string]: any;
}
export interface Options extends Array<Option> {
}
export interface OptionProps {
    className?: string;
    multi?: boolean;
    multiple?: boolean;
    valueField?: string;
    labelField?: string;
    simpleValue?: boolean;
    options: Options;
    joinValues?: boolean;
    extractValue?: boolean;
    delimiter?: string;
    clearable?: boolean;
    placeholder?: string;
    disabled?: boolean;
    creatable?: boolean;
    onAdd?: (idx?: number | Array<number>, value?: any, skipForm?: boolean) => void;
    editable?: boolean;
    onEdit?: (value: Option, origin?: Option, skipForm?: boolean) => void;
    removable?: boolean;
    onDelete?: (value: Option) => void;
}
export declare type OptionValue = string | number | null | undefined | Option;
export declare function value2array(value: OptionValue | Array<OptionValue>, props: Pick<OptionProps, 'multi' | 'multiple' | 'delimiter' | 'valueField' | 'options'>): Array<Option>;
export declare function expandValue(value: OptionValue, options: Options, valueField?: string): Option | null;
export declare function matchOptionValue(a: OptionValue, b: Option, valueField?: string): boolean;
export declare function optionValueCompare(a: OptionValue, valueField?: string): (b: Option) => boolean;
export declare function normalizeOptions(options: string | {
    [propName: string]: string;
} | Array<string> | Options, share?: {
    values: Array<any>;
    options: Array<any>;
}): Options;
interface SelectProps extends OptionProps, ThemeProps, LocaleProps {
    className?: string;
    creatable: boolean;
    createBtnLabel: string;
    multiple: boolean;
    valueField: string;
    labelField: string;
    searchable?: boolean;
    options: Array<Option>;
    value: any;
    loadOptions?: Function;
    searchPromptText: string;
    loading?: boolean;
    loadingPlaceholder: string;
    spinnerClassName?: string;
    noResultsText: string;
    clearable: boolean;
    clearAllText: string;
    clearValueText: string;
    placeholder: string;
    inline: boolean;
    disabled: boolean;
    popOverContainer?: any;
    overlayPlacement?: string;
    onChange: (value: void | string | Option | Array<Option>) => void;
    onFocus?: Function;
    onBlur?: Function;
    checkAll?: boolean;
    checkAllLabel?: string;
    defaultCheckAll?: boolean;
    simpleValue?: boolean;
    defaultOpen?: boolean;
}
interface SelectState {
    itemHeight: number;
    isOpen: boolean;
    isFocused: boolean;
    inputValue: string;
    highlightedIndex: number;
    selection: Array<Option>;
}
export declare class Select extends React.Component<SelectProps, SelectState> {
    static defaultProps: {
        multiple: boolean;
        clearable: boolean;
        creatable: boolean;
        createBtnLabel: string;
        searchPromptText: string;
        loadingPlaceholder: string;
        noResultsText: string;
        clearAllText: string;
        clearValueText: string;
        placeholder: string;
        valueField: string;
        labelField: string;
        inline: boolean;
        disabled: boolean;
        checkAll: boolean;
        checkAllLabel: string;
        defaultCheckAll: boolean;
        overlayPlacement: string;
    };
    input: HTMLInputElement;
    target: HTMLElement;
    menu: React.RefObject<HTMLDivElement>;
    constructor(props: SelectProps);
    componentDidMount(): void;
    componentDidUpdate(prevProps: SelectProps): void;
    open(): void;
    close(): void;
    toggle(e?: React.MouseEvent<HTMLDivElement>): void;
    onFocus(e: any): void;
    onBlur(e: any): void;
    focus(): void;
    blur(): void;
    getTarget(): HTMLElement;
    inputRef(ref: HTMLInputElement): void;
    toggleCheckAll(): void;
    removeItem(index: number, e?: React.MouseEvent<HTMLElement>): void;
    handleInputChange(evt: React.ChangeEvent<HTMLInputElement>): void;
    handleChange(selectItem: any): void;
    handleStateChange(changes: any): void;
    handleKeyPress(e: React.KeyboardEvent): void;
    clearValue(e: React.MouseEvent<any>): void;
    handleAddClick(): void;
    handleEditClick(e: Event, item: any): void;
    handleDeleteClick(e: Event, item: any): void;
    menuItemRef(ref: any): void;
    renderValue({ inputValue, isOpen }: ControllerStateAndHelpers<any>): JSX.Element | JSX.Element[];
    renderOuter({ selectedItem, getItemProps, highlightedIndex, inputValue, isOpen, getToggleButtonProps, getInputProps }: ControllerStateAndHelpers<any>): JSX.Element;
    render(): JSX.Element;
}
declare const _default: {
    new (props: Readonly<Pick<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
        multiple: boolean;
        clearable: boolean;
        creatable: boolean;
        createBtnLabel: string;
        searchPromptText: string;
        loadingPlaceholder: string;
        noResultsText: string;
        clearAllText: string;
        clearValueText: string;
        placeholder: string;
        valueField: string;
        labelField: string;
        inline: boolean;
        disabled: boolean;
        checkAll: boolean;
        checkAllLabel: string;
        defaultCheckAll: boolean;
        overlayPlacement: string;
    }, never>> & {
        locale?: string | undefined;
        translate?: ((str: string, ...args: any[]) => string) | undefined;
    }, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "inline" | "disabled" | "locale" | "translate" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen"> & {
        theme?: string | undefined;
        className?: string | undefined;
        classPrefix?: string | undefined;
        classnames?: ClassNamesFn | undefined;
    }>): {
        render(): JSX.Element;
        context: any;
        setState<K extends never>(state: {} | ((prevState: Readonly<{}>, props: Readonly<Pick<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "inline" | "disabled" | "locale" | "translate" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen"> & {
            theme?: string | undefined;
            className?: string | undefined;
            classPrefix?: string | undefined;
            classnames?: ClassNamesFn | undefined;
        }>) => {} | Pick<{}, K> | null) | Pick<{}, K> | null, callback?: (() => void) | undefined): void;
        forceUpdate(callback?: (() => void) | undefined): void;
        readonly props: Readonly<Pick<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "inline" | "disabled" | "locale" | "translate" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen"> & {
            theme?: string | undefined;
            className?: string | undefined;
            classPrefix?: string | undefined;
            classnames?: ClassNamesFn | undefined;
        }> & Readonly<{
            children?: React.ReactNode;
        }>;
        state: Readonly<{}>;
        refs: {
            [key: string]: React.ReactInstance;
        };
        componentDidMount?(): void;
        shouldComponentUpdate?(nextProps: Readonly<Pick<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "inline" | "disabled" | "locale" | "translate" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen"> & {
            theme?: string | undefined;
            className?: string | undefined;
            classPrefix?: string | undefined;
            classnames?: ClassNamesFn | undefined;
        }>, nextState: Readonly<{}>, nextContext: any): boolean;
        componentWillUnmount?(): void;
        componentDidCatch?(error: Error, errorInfo: React.ErrorInfo): void;
        getSnapshotBeforeUpdate?(prevProps: Readonly<Pick<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "inline" | "disabled" | "locale" | "translate" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen"> & {
            theme?: string | undefined;
            className?: string | undefined;
            classPrefix?: string | undefined;
            classnames?: ClassNamesFn | undefined;
        }>, prevState: Readonly<{}>): any;
        componentDidUpdate?(prevProps: Readonly<Pick<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "inline" | "disabled" | "locale" | "translate" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen"> & {
            theme?: string | undefined;
            className?: string | undefined;
            classPrefix?: string | undefined;
            classnames?: ClassNamesFn | undefined;
        }>, prevState: Readonly<{}>, snapshot?: any): void;
        componentWillMount?(): void;
        UNSAFE_componentWillMount?(): void;
        componentWillReceiveProps?(nextProps: Readonly<Pick<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "inline" | "disabled" | "locale" | "translate" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen"> & {
            theme?: string | undefined;
            className?: string | undefined;
            classPrefix?: string | undefined;
            classnames?: ClassNamesFn | undefined;
        }>, nextContext: any): void;
        UNSAFE_componentWillReceiveProps?(nextProps: Readonly<Pick<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "inline" | "disabled" | "locale" | "translate" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen"> & {
            theme?: string | undefined;
            className?: string | undefined;
            classPrefix?: string | undefined;
            classnames?: ClassNamesFn | undefined;
        }>, nextContext: any): void;
        componentWillUpdate?(nextProps: Readonly<Pick<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "inline" | "disabled" | "locale" | "translate" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen"> & {
            theme?: string | undefined;
            className?: string | undefined;
            classPrefix?: string | undefined;
            classnames?: ClassNamesFn | undefined;
        }>, nextState: Readonly<{}>, nextContext: any): void;
        UNSAFE_componentWillUpdate?(nextProps: Readonly<Pick<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "inline" | "disabled" | "locale" | "translate" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen"> & {
            theme?: string | undefined;
            className?: string | undefined;
            classPrefix?: string | undefined;
            classnames?: ClassNamesFn | undefined;
        }>, nextState: Readonly<{}>, nextContext: any): void;
    };
    new (props: Pick<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
        multiple: boolean;
        clearable: boolean;
        creatable: boolean;
        createBtnLabel: string;
        searchPromptText: string;
        loadingPlaceholder: string;
        noResultsText: string;
        clearAllText: string;
        clearValueText: string;
        placeholder: string;
        valueField: string;
        labelField: string;
        inline: boolean;
        disabled: boolean;
        checkAll: boolean;
        checkAllLabel: string;
        defaultCheckAll: boolean;
        overlayPlacement: string;
    }, never>> & {
        locale?: string | undefined;
        translate?: ((str: string, ...args: any[]) => string) | undefined;
    }, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "inline" | "disabled" | "locale" | "translate" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen"> & {
        theme?: string | undefined;
        className?: string | undefined;
        classPrefix?: string | undefined;
        classnames?: ClassNamesFn | undefined;
    }, context?: any): {
        render(): JSX.Element;
        context: any;
        setState<K extends never>(state: {} | ((prevState: Readonly<{}>, props: Readonly<Pick<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "inline" | "disabled" | "locale" | "translate" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen"> & {
            theme?: string | undefined;
            className?: string | undefined;
            classPrefix?: string | undefined;
            classnames?: ClassNamesFn | undefined;
        }>) => {} | Pick<{}, K> | null) | Pick<{}, K> | null, callback?: (() => void) | undefined): void;
        forceUpdate(callback?: (() => void) | undefined): void;
        readonly props: Readonly<Pick<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "inline" | "disabled" | "locale" | "translate" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen"> & {
            theme?: string | undefined;
            className?: string | undefined;
            classPrefix?: string | undefined;
            classnames?: ClassNamesFn | undefined;
        }> & Readonly<{
            children?: React.ReactNode;
        }>;
        state: Readonly<{}>;
        refs: {
            [key: string]: React.ReactInstance;
        };
        componentDidMount?(): void;
        shouldComponentUpdate?(nextProps: Readonly<Pick<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "inline" | "disabled" | "locale" | "translate" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen"> & {
            theme?: string | undefined;
            className?: string | undefined;
            classPrefix?: string | undefined;
            classnames?: ClassNamesFn | undefined;
        }>, nextState: Readonly<{}>, nextContext: any): boolean;
        componentWillUnmount?(): void;
        componentDidCatch?(error: Error, errorInfo: React.ErrorInfo): void;
        getSnapshotBeforeUpdate?(prevProps: Readonly<Pick<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "inline" | "disabled" | "locale" | "translate" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen"> & {
            theme?: string | undefined;
            className?: string | undefined;
            classPrefix?: string | undefined;
            classnames?: ClassNamesFn | undefined;
        }>, prevState: Readonly<{}>): any;
        componentDidUpdate?(prevProps: Readonly<Pick<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "inline" | "disabled" | "locale" | "translate" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen"> & {
            theme?: string | undefined;
            className?: string | undefined;
            classPrefix?: string | undefined;
            classnames?: ClassNamesFn | undefined;
        }>, prevState: Readonly<{}>, snapshot?: any): void;
        componentWillMount?(): void;
        UNSAFE_componentWillMount?(): void;
        componentWillReceiveProps?(nextProps: Readonly<Pick<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "inline" | "disabled" | "locale" | "translate" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen"> & {
            theme?: string | undefined;
            className?: string | undefined;
            classPrefix?: string | undefined;
            classnames?: ClassNamesFn | undefined;
        }>, nextContext: any): void;
        UNSAFE_componentWillReceiveProps?(nextProps: Readonly<Pick<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "inline" | "disabled" | "locale" | "translate" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen"> & {
            theme?: string | undefined;
            className?: string | undefined;
            classPrefix?: string | undefined;
            classnames?: ClassNamesFn | undefined;
        }>, nextContext: any): void;
        componentWillUpdate?(nextProps: Readonly<Pick<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "inline" | "disabled" | "locale" | "translate" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen"> & {
            theme?: string | undefined;
            className?: string | undefined;
            classPrefix?: string | undefined;
            classnames?: ClassNamesFn | undefined;
        }>, nextState: Readonly<{}>, nextContext: any): void;
        UNSAFE_componentWillUpdate?(nextProps: Readonly<Pick<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "inline" | "disabled" | "locale" | "translate" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen"> & {
            theme?: string | undefined;
            className?: string | undefined;
            classPrefix?: string | undefined;
            classnames?: ClassNamesFn | undefined;
        }>, nextState: Readonly<{}>, nextContext: any): void;
    };
    displayName: string;
    contextType: React.Context<string>;
    ComposedComponent: {
        new (props: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }>): {
            render(): JSX.Element;
            context: any;
            setState<K_1 extends never>(state: {} | ((prevState: Readonly<{}>, props: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }>) => {} | Pick<{}, K_1> | null) | Pick<{}, K_1> | null, callback?: (() => void) | undefined): void;
            forceUpdate(callback?: (() => void) | undefined): void;
            readonly props: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }> & Readonly<{
                children?: React.ReactNode;
            }>;
            state: Readonly<{}>;
            refs: {
                [key: string]: React.ReactInstance;
            };
            componentDidMount?(): void;
            shouldComponentUpdate?(nextProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }>, nextState: Readonly<{}>, nextContext: any): boolean;
            componentWillUnmount?(): void;
            componentDidCatch?(error: Error, errorInfo: React.ErrorInfo): void;
            getSnapshotBeforeUpdate?(prevProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }>, prevState: Readonly<{}>): any;
            componentDidUpdate?(prevProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }>, prevState: Readonly<{}>, snapshot?: any): void;
            componentWillMount?(): void;
            UNSAFE_componentWillMount?(): void;
            componentWillReceiveProps?(nextProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }>, nextContext: any): void;
            UNSAFE_componentWillReceiveProps?(nextProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }>, nextContext: any): void;
            componentWillUpdate?(nextProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }>, nextState: Readonly<{}>, nextContext: any): void;
            UNSAFE_componentWillUpdate?(nextProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }>, nextState: Readonly<{}>, nextContext: any): void;
        };
        new (props: Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }, context?: any): {
            render(): JSX.Element;
            context: any;
            setState<K_1 extends never>(state: {} | ((prevState: Readonly<{}>, props: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }>) => {} | Pick<{}, K_1> | null) | Pick<{}, K_1> | null, callback?: (() => void) | undefined): void;
            forceUpdate(callback?: (() => void) | undefined): void;
            readonly props: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }> & Readonly<{
                children?: React.ReactNode;
            }>;
            state: Readonly<{}>;
            refs: {
                [key: string]: React.ReactInstance;
            };
            componentDidMount?(): void;
            shouldComponentUpdate?(nextProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }>, nextState: Readonly<{}>, nextContext: any): boolean;
            componentWillUnmount?(): void;
            componentDidCatch?(error: Error, errorInfo: React.ErrorInfo): void;
            getSnapshotBeforeUpdate?(prevProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }>, prevState: Readonly<{}>): any;
            componentDidUpdate?(prevProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }>, prevState: Readonly<{}>, snapshot?: any): void;
            componentWillMount?(): void;
            UNSAFE_componentWillMount?(): void;
            componentWillReceiveProps?(nextProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }>, nextContext: any): void;
            UNSAFE_componentWillReceiveProps?(nextProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }>, nextContext: any): void;
            componentWillUpdate?(nextProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }>, nextState: Readonly<{}>, nextContext: any): void;
            UNSAFE_componentWillUpdate?(nextProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }>, nextState: Readonly<{}>, nextContext: any): void;
        };
        displayName: string;
        contextType: React.Context<string>;
        ComposedComponent: typeof Select;
    } & import("hoist-non-react-statics").NonReactStatics<typeof Select, {}> & {
        ComposedComponent: typeof Select;
    };
} & import("hoist-non-react-statics").NonReactStatics<{
    new (props: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
        multiple: boolean;
        clearable: boolean;
        creatable: boolean;
        createBtnLabel: string;
        searchPromptText: string;
        loadingPlaceholder: string;
        noResultsText: string;
        clearAllText: string;
        clearValueText: string;
        placeholder: string;
        valueField: string;
        labelField: string;
        inline: boolean;
        disabled: boolean;
        checkAll: boolean;
        checkAllLabel: string;
        defaultCheckAll: boolean;
        overlayPlacement: string;
    }, never>> & {
        locale?: string | undefined;
        translate?: ((str: string, ...args: any[]) => string) | undefined;
    }>): {
        render(): JSX.Element;
        context: any;
        setState<K_1 extends never>(state: {} | ((prevState: Readonly<{}>, props: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }>) => {} | Pick<{}, K_1> | null) | Pick<{}, K_1> | null, callback?: (() => void) | undefined): void;
        forceUpdate(callback?: (() => void) | undefined): void;
        readonly props: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }> & Readonly<{
            children?: React.ReactNode;
        }>;
        state: Readonly<{}>;
        refs: {
            [key: string]: React.ReactInstance;
        };
        componentDidMount?(): void;
        shouldComponentUpdate?(nextProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }>, nextState: Readonly<{}>, nextContext: any): boolean;
        componentWillUnmount?(): void;
        componentDidCatch?(error: Error, errorInfo: React.ErrorInfo): void;
        getSnapshotBeforeUpdate?(prevProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }>, prevState: Readonly<{}>): any;
        componentDidUpdate?(prevProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }>, prevState: Readonly<{}>, snapshot?: any): void;
        componentWillMount?(): void;
        UNSAFE_componentWillMount?(): void;
        componentWillReceiveProps?(nextProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }>, nextContext: any): void;
        UNSAFE_componentWillReceiveProps?(nextProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }>, nextContext: any): void;
        componentWillUpdate?(nextProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }>, nextState: Readonly<{}>, nextContext: any): void;
        UNSAFE_componentWillUpdate?(nextProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }>, nextState: Readonly<{}>, nextContext: any): void;
    };
    new (props: Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
        multiple: boolean;
        clearable: boolean;
        creatable: boolean;
        createBtnLabel: string;
        searchPromptText: string;
        loadingPlaceholder: string;
        noResultsText: string;
        clearAllText: string;
        clearValueText: string;
        placeholder: string;
        valueField: string;
        labelField: string;
        inline: boolean;
        disabled: boolean;
        checkAll: boolean;
        checkAllLabel: string;
        defaultCheckAll: boolean;
        overlayPlacement: string;
    }, never>> & {
        locale?: string | undefined;
        translate?: ((str: string, ...args: any[]) => string) | undefined;
    }, context?: any): {
        render(): JSX.Element;
        context: any;
        setState<K_1 extends never>(state: {} | ((prevState: Readonly<{}>, props: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }>) => {} | Pick<{}, K_1> | null) | Pick<{}, K_1> | null, callback?: (() => void) | undefined): void;
        forceUpdate(callback?: (() => void) | undefined): void;
        readonly props: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }> & Readonly<{
            children?: React.ReactNode;
        }>;
        state: Readonly<{}>;
        refs: {
            [key: string]: React.ReactInstance;
        };
        componentDidMount?(): void;
        shouldComponentUpdate?(nextProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }>, nextState: Readonly<{}>, nextContext: any): boolean;
        componentWillUnmount?(): void;
        componentDidCatch?(error: Error, errorInfo: React.ErrorInfo): void;
        getSnapshotBeforeUpdate?(prevProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }>, prevState: Readonly<{}>): any;
        componentDidUpdate?(prevProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }>, prevState: Readonly<{}>, snapshot?: any): void;
        componentWillMount?(): void;
        UNSAFE_componentWillMount?(): void;
        componentWillReceiveProps?(nextProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }>, nextContext: any): void;
        UNSAFE_componentWillReceiveProps?(nextProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }>, nextContext: any): void;
        componentWillUpdate?(nextProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }>, nextState: Readonly<{}>, nextContext: any): void;
        UNSAFE_componentWillUpdate?(nextProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }>, nextState: Readonly<{}>, nextContext: any): void;
    };
    displayName: string;
    contextType: React.Context<string>;
    ComposedComponent: typeof Select;
} & import("hoist-non-react-statics").NonReactStatics<typeof Select, {}> & {
    ComposedComponent: typeof Select;
}, {}> & {
    ComposedComponent: {
        new (props: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }>): {
            render(): JSX.Element;
            context: any;
            setState<K_1 extends never>(state: {} | ((prevState: Readonly<{}>, props: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }>) => {} | Pick<{}, K_1> | null) | Pick<{}, K_1> | null, callback?: (() => void) | undefined): void;
            forceUpdate(callback?: (() => void) | undefined): void;
            readonly props: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }> & Readonly<{
                children?: React.ReactNode;
            }>;
            state: Readonly<{}>;
            refs: {
                [key: string]: React.ReactInstance;
            };
            componentDidMount?(): void;
            shouldComponentUpdate?(nextProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }>, nextState: Readonly<{}>, nextContext: any): boolean;
            componentWillUnmount?(): void;
            componentDidCatch?(error: Error, errorInfo: React.ErrorInfo): void;
            getSnapshotBeforeUpdate?(prevProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }>, prevState: Readonly<{}>): any;
            componentDidUpdate?(prevProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }>, prevState: Readonly<{}>, snapshot?: any): void;
            componentWillMount?(): void;
            UNSAFE_componentWillMount?(): void;
            componentWillReceiveProps?(nextProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }>, nextContext: any): void;
            UNSAFE_componentWillReceiveProps?(nextProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }>, nextContext: any): void;
            componentWillUpdate?(nextProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }>, nextState: Readonly<{}>, nextContext: any): void;
            UNSAFE_componentWillUpdate?(nextProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }>, nextState: Readonly<{}>, nextContext: any): void;
        };
        new (props: Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
            multiple: boolean;
            clearable: boolean;
            creatable: boolean;
            createBtnLabel: string;
            searchPromptText: string;
            loadingPlaceholder: string;
            noResultsText: string;
            clearAllText: string;
            clearValueText: string;
            placeholder: string;
            valueField: string;
            labelField: string;
            inline: boolean;
            disabled: boolean;
            checkAll: boolean;
            checkAllLabel: string;
            defaultCheckAll: boolean;
            overlayPlacement: string;
        }, never>> & {
            locale?: string | undefined;
            translate?: ((str: string, ...args: any[]) => string) | undefined;
        }, context?: any): {
            render(): JSX.Element;
            context: any;
            setState<K_1 extends never>(state: {} | ((prevState: Readonly<{}>, props: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }>) => {} | Pick<{}, K_1> | null) | Pick<{}, K_1> | null, callback?: (() => void) | undefined): void;
            forceUpdate(callback?: (() => void) | undefined): void;
            readonly props: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }> & Readonly<{
                children?: React.ReactNode;
            }>;
            state: Readonly<{}>;
            refs: {
                [key: string]: React.ReactInstance;
            };
            componentDidMount?(): void;
            shouldComponentUpdate?(nextProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }>, nextState: Readonly<{}>, nextContext: any): boolean;
            componentWillUnmount?(): void;
            componentDidCatch?(error: Error, errorInfo: React.ErrorInfo): void;
            getSnapshotBeforeUpdate?(prevProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }>, prevState: Readonly<{}>): any;
            componentDidUpdate?(prevProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }>, prevState: Readonly<{}>, snapshot?: any): void;
            componentWillMount?(): void;
            UNSAFE_componentWillMount?(): void;
            componentWillReceiveProps?(nextProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }>, nextContext: any): void;
            UNSAFE_componentWillReceiveProps?(nextProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }>, nextContext: any): void;
            componentWillUpdate?(nextProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }>, nextState: Readonly<{}>, nextContext: any): void;
            UNSAFE_componentWillUpdate?(nextProps: Readonly<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "value" | "loading" | "delimiter" | "joinValues" | "extractValue" | "options" | "className" | "classPrefix" | "classnames" | "theme" | "onFocus" | "onBlur" | "onChange" | "popOverContainer" | "spinnerClassName" | "multi" | "simpleValue" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "searchable" | "loadOptions" | "defaultOpen"> & Partial<Pick<Pick<SelectProps, "value" | "loading" | "multiple" | "delimiter" | "valueField" | "labelField" | "joinValues" | "extractValue" | "options" | "className" | "inline" | "disabled" | "classPrefix" | "classnames" | "theme" | "placeholder" | "onFocus" | "onBlur" | "onChange" | "clearable" | "popOverContainer" | "overlayPlacement" | "spinnerClassName" | "multi" | "simpleValue" | "creatable" | "onAdd" | "editable" | "onEdit" | "removable" | "onDelete" | "noResultsText" | "createBtnLabel" | "searchable" | "loadOptions" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll" | "defaultOpen">, "multiple" | "valueField" | "labelField" | "inline" | "disabled" | "placeholder" | "clearable" | "overlayPlacement" | "creatable" | "noResultsText" | "createBtnLabel" | "searchPromptText" | "loadingPlaceholder" | "clearAllText" | "clearValueText" | "checkAll" | "checkAllLabel" | "defaultCheckAll">> & Partial<Pick<{
                multiple: boolean;
                clearable: boolean;
                creatable: boolean;
                createBtnLabel: string;
                searchPromptText: string;
                loadingPlaceholder: string;
                noResultsText: string;
                clearAllText: string;
                clearValueText: string;
                placeholder: string;
                valueField: string;
                labelField: string;
                inline: boolean;
                disabled: boolean;
                checkAll: boolean;
                checkAllLabel: string;
                defaultCheckAll: boolean;
                overlayPlacement: string;
            }, never>> & {
                locale?: string | undefined;
                translate?: ((str: string, ...args: any[]) => string) | undefined;
            }>, nextState: Readonly<{}>, nextContext: any): void;
        };
        displayName: string;
        contextType: React.Context<string>;
        ComposedComponent: typeof Select;
    } & import("hoist-non-react-statics").NonReactStatics<typeof Select, {}> & {
        ComposedComponent: typeof Select;
    };
};
export default _default;
